dofile("data/scripts/game_helpers.lua")
dofile_once("data/scripts/lib/utilities.lua")
dofile("data/selectable_classes/classes/class_list.lua")
dofile("data/selectable_classes/classes/spawn_classes.lua")
dofile("data/selectable_classes/util.lua")

function give_loadout(entity, data, x, y)
	if not x then x = 0 end
	if not y then y = 0 end
	--give perks
	if data.perks ~= nil then
		for i,perk in ipairs(data.perks) do
			give_perk(entity,perk)
		end
	end

	local inventory = get_entity_child(entity, "inventory_quick")

	if inventory ~= nil then
		--give wands
		if data.wands ~= nil then
			for i,gun in ipairs(data.wands) do
				local wand
				if type(gun) == "table" then
					wand = build_wand(gun, x, y)
				else
					wand = EntityLoad(gun, x, y)
				end 
				if wand ~= nil then
					if gun.actions ~= nil then
						for j,action in ipairs(gun.actions) do
							AddGunAction(wand,action)
						end
					end
					EntityAddChild(inventory, wand)
				end
			end
		end
		
		--give items
		if data.items ~= nil then
			for i,item in ipairs(data.items) do
				local entity = EntityLoad(item.base, x, y)
				if entity ~= nil then
					if item.setup ~= nil then item.setup(entity) end
					EntityAddChild(inventory, entity)
				end
			end
		end
	end
end

function item_pickup(entity_item, entity_who_picked, item_name)
	--find class data
	local class_id = "NONE"
	local class_storage = {}
	local storage_comps = EntityGetComponent(entity_item, "VariableStorageComponent")
	if storage_comps == nil then return end
	for i,comp in ipairs(storage_comps) do
		local comp_name = ComponentGetValue(comp, "name")
		if comp_name == "class_id" then
			class_id = ComponentGetValue(comp, "value_string")
		else
			class_storage[comp_name] = tonumber(ComponentGetValue(comp, "value_int"))
		end
	end

	if class_id == "page" then
		--destroy classes
		local all_classes = EntityGetWithTag("player_class")
		if #all_classes > 0 then
			for i,class_pickup in ipairs(all_classes) do
				EntityKill(class_pickup)
			end
		else EntityKill(entity_item) end
		return spawn_classes(class_storage.class_x, class_storage.class_y, class_storage.class_index)
	end

	local class_name = "CLASS_NAME_NOT_DEFINED"
	local class_desc = "CLASS_DESCRIPTION_NOT_DEFINED"
	edit_component(entity_item, "ItemComponent", function(comp,vars)
		class_name = ComponentGetValue(comp, "item_name")
		class_desc = ComponentGetValue(comp, "ui_description")
	end)

	local class_data = get_class_with_id(class_list, class_id)
	if class_data == nil then return end

	--TODO: create/pick UI icons for selected classes
	--[[local entity_ui = EntityCreateNew("")
	EntityAddComponent(entity_ui, "UIIconComponent", { 
		name = class_data.ui_name,
		description = class_data.ui_description,
		icon_sprite_file = class_data.ui_icon
	})
	EntityAddChild(entity_who_picked, entity_ui)]]

	local x, y = EntityGetTransform(entity_who_picked)
	give_loadout(entity_who_picked, class_data, x, y)
	if class_list.standard_items then give_loadout(entity_who_picked, class_list.standard_items, -1500, -300) end
	EntityRemoveComponent(entity_who_picked, GameGetGameEffect(entity_who_picked, "MOVEMENT_FASTER"))

	--do effects
	--local pos_x, pos_y = EntityGetTransform(entity_item)
	--EntityLoad( "data/entities/particles/image_emitters/perk_effect_pacifist.xml", pos_x, pos_y )
	GamePrintImportant(GameTextGet("$log_pickedup_perk", GameTextGetTranslatedOrNot(class_name)), class_desc)
	
	--destroy classes
	local all_classes = EntityGetWithTag("player_class")
	if #all_classes > 0 then
		for i,class_pickup in ipairs(all_classes) do
			EntityKill(class_pickup)
		end
	else EntityKill(entity_item) end

	--spawn portal
	EntityLoad("data/selectable_classes/teleport_start.xml", x, y)
end