dofile("data/selectable_classes/classes/class_list.lua")

local room_width = 220
local class_vert_gap = 100
local class_horiz_gap = 20
local class_per_floor = room_width / class_horiz_gap + 1
local room_floors = 1
local class_per_room = class_per_floor * room_floors

function spawn_classes(x, y, start)
    local finish = math.min(start + class_per_room - 1, #class_list)
    local num_classes = finish-start
    local needs_prev_page = start > 1
    local needs_next_page = finish < #class_list
    if needs_prev_page then finish = finish - 1 end
    if needs_next_page then finish = finish - 1 end

    --generate class pickups in rows
    for i=0,num_classes do
        --load pickup
        local entity = EntityLoad("data/selectable_classes/classes/class.xml", x + class_horiz_gap * (i % class_per_floor), y + class_vert_gap * math.floor(i/class_per_floor))
        if entity == nil then return end

        local class

        if needs_prev_page and i == 0 then 
            class = {
                id = "page",
                ui_name = "Previous Page",
                pickup_icon = "data/selectable_classes/classes/assets/prev_page.png",
            }
        elseif needs_next_page and i == num_classes then
            class = {
                id = "page",
                ui_name = "Next Page",
                pickup_icon = "data/selectable_classes/classes/assets/next_page.png",
            }
        else
            class = class_list[start + i - (needs_prev_page and 1 or 0)]
        end

        --setup components
        EntityAddComponent(entity, "SpriteComponent", { 
            image_file = class.pickup_icon or "data/items_gfx/perk.xml",  
            offset_x = "8", 
            offset_y = "8", 
            update_transform = "1",
            update_transform_rotation = "0",
        })
        EntityAddComponent(entity, "UIInfoComponent", { 
            name = class.ui_name,
        })
        EntityAddComponent(entity, "ItemComponent", { 
            item_name = class.ui_name,
            ui_description = class.ui_description or "",
            play_spinning_animation = "0",
            play_hover_animation = "0",
            play_pick_sound = "0",
        })
        EntityAddComponent( entity, "SpriteOffsetAnimatorComponent", { 
            sprite_id="-1",
            x_amount="0",
            x_phase="0",
            x_phase_offset="0",
            x_speed="0",
            y_amount="2",
            y_speed="3",
        })
        EntityAddComponent( entity, "VariableStorageComponent", { 
            name = "class_id",
            value_string = class.id,
        })
        --if this isn't a class, add coordinate & index storage to the pickup for pagination
        if class.id == "page" then
            EntityAddComponent( entity, "VariableStorageComponent", { 
                name = "class_x",
                value_int = x,
            })
            EntityAddComponent( entity, "VariableStorageComponent", { 
                name = "class_y",
                value_int = y,
            })
            local index = i==0 and math.max(start-class_per_room+2,1) or finish+1
            if index == 2 then index = 1 end --ugh
            EntityAddComponent( entity, "VariableStorageComponent", { 
                name = "class_index",
                value_int = index,
            })
        end
    end
end