dofile("data/scripts/gun/procedural/gun_procedural.lua")
dofile("data/scripts/perks/perk.lua")
dofile_once("data/scripts/gun/procedural/wands.lua")

--mapping of the table keys used by gun_procedural.lua to abilitycomponent structure
local wand_schema = {
    "ui_name",
    "mana_charge_speed",
    "mana_max",
    gun_config = { "actions_per_round", "reload_time", "deck_capacity", "shuffle_deck_when_empty" },
    gunaction_config = { "fire_rate_wait", "spread_degrees", "speed_multiplier" }
}

--create and return a wand built from gun data as in gun_procedural.lua
function build_wand(data, x, y)
    local entity = EntityLoad("data/selectable_classes/classes/wand_empty.xml", x, y)
	local ability = EntityGetFirstComponent(entity, "AbilityComponent")

    if data.always_cast ~= nil then
        local always_cast = data.always_cast
        if type(data.always_cast) ~= "table" then always_cast = { data.always_cast } end
        for i,action in ipairs(always_cast) do
            AddGunActionPermanent(entity, action)
        end
        data.deck_capacity = data.deck_capacity + #always_cast
    end

    --iterate over the wand_schema and set values on the abilitycomponent
    function wand_set_values(obj, name)
        for k,v in pairs(obj) do
            if type(v) == "table" then wand_set_values(v,k)
            else
                if name == nil then ComponentSetValue(ability, v, tostring(data[v]))
                else ComponentObjectSetValue(ability, name, v, tostring(data[v])) end
            end
        end
    end
    wand_set_values(wand_schema)
    ComponentSetValue(ability, "mana", data.mana_max)

    local wand = data.sprite and wands[data.sprite] or GetWand(data)
    if wand then SetWandSprite(entity, ability, wand.file, wand.grip_x, wand.grip_y, (wand.tip_x - wand.grip_x), (wand.tip_y - wand.grip_y)) end

    local x, y = EntityGetTransform(entity)
    return entity
end

function give_perk(target, perk)
    local data = get_perk_with_id(perk_list, perk)

    --apply game effect
    if data.game_effect ~= nil then
        local game_effect_comp = GetGameEffectLoadTo(target, data.game_effect, true)
        if game_effect_comp ~= nil then
            ComponentSetValue(game_effect_comp, "frames", "-1")
        end
    end

    --call extra function
    if data.func ~= nil then
        --these aren't the right arguments, but none of the perks
        --use entity_perk_item or item_name anyway (why would they...?)
        data.func(target, target, perk)
    end

    --add to UI
    local perk_ui = EntityCreateNew("")
    EntityAddComponent(perk_ui, "UIIconComponent", { 
        name = data.ui_name,
        description = data.ui_description,
        icon_sprite_file = data.ui_icon
    })
    EntityAddChild(target, perk_ui)
end

function get_entity_child(entity, name)
    local children = EntityGetAllChildren(entity)
    if children == nil then return end
    for i,child in ipairs(children) do
        if EntityGetName(child) == name then
            return child
        end
    end
end

function get_player()
    return EntityGetWithTag("player_unit")[1]
end

function get_entity_pos(entity)
    return EntityGetTransform(entity)
end
